package cn.com.qmth.examcloud.support;

/**
 * 缓存相关常量
 * 注：cache key 命名规则
 * $模块:关键字:参数1_参数2  示例：$OE:XXX:123_abc
 */
public interface CacheConstants {

    /* #################### 登录会话、锁等KEY #################### */

    /**
     * 缓存 - 登录会话前缀：{userType}_{rootOrgId}_{userId}
     */
    String CACHE_LOGIN_SESSION = "$SS:";

    /**
     * 缓存 - APP登录会话前缀：{CACHE_LOGIN_SESSION}
     */
    String CACHE_APP_LOGIN_SESSION = "$APP:";

    /**
     * 缓存 - 异常状态：{cacheKey}
     */
    String CACHE_EXCEPTION = "$CACHE_EXCEPTION:";

    /**
     * 缓存 - 锁前缀
     */
    String LOCK_PREFIX = "$LOCK:";

    /**
     * 全局请求锁：{mapping}
     */
    String LOCK_REQUEST_GLOBAL = LOCK_PREFIX + "REQ_GLOBAL:";

    /**
     * 会话请求锁：{userKey}
     */
    String LOCK_REQUEST_SESSION = LOCK_PREFIX + "REQ_SESSION:";

    /**
     * 考试控制锁：{studentId}
     */
    String LOCK_EXAM_CONTROL = LOCK_PREFIX + "OE_EXAM_CONTROL:";

    /**
     * 考试交卷锁：{examRecordDataId}
     */
    String LOCK_HAND_IN_EXAM = LOCK_PREFIX + "OE_HAND_IN_EXAM:";

    /**
     * 人脸活体检测信息锁：{studentId}
     */
    String LOCK_GET_FACE_BIOPSY = LOCK_PREFIX + "OE_GET_FACE_BIOPSY:";

    /**
     * 人脸比对信息锁：{examRecordDataId}
     */
    String LOCK_FACE_COMPARE = LOCK_PREFIX + "OE_FACE_COMPARE:";

    /**
     * 考试记录清理锁
     */
    String LOCK_EXAM_DATA_CLEAN = LOCK_PREFIX + "OE_EXAM_DATA_CLEAN";

    /**
     * 网考导出任务锁：{exportTaskId}
     */
    String LOCK_OE_EXPORT_TASK = LOCK_PREFIX + "OE_EXPORT_TASK:";

    /**
     * 阅卷导出任务锁：{exportTaskId}
     */
    String LOCK_M_EXPORT_TASK = LOCK_PREFIX + "M_EXPORT_TASK:";

    /**
     * 考试更新锁：{rootOrgId}_{examCode}
     */
    String LOCK_E_EXAM_SAVE = LOCK_PREFIX + "E_EXAM_SAVE:";

    /**
     * 考试设置更新锁：{examId}_{orgId}_{courseId}_{studentId}
     */
    String LOCK_E_EXAM_SPECIAL_SETTINGS = LOCK_PREFIX + "E_EXAM_SPECIAL_SETTINGS:";



    /* #################### 基础模块 #################### */

    /**
     * 缓存 - 应用模块信息：{appId}
     */
    String CACHE_B_APP = "$B:APP:";

    /**
     * 缓存 - 登录规则：{loginRuleType}
     */
    String CACHE_B_LOGIN_RULE = "$B:LOGIN_RULE:";

    /**
     * 缓存 - 加密组合配置
     */
    String CACHE_B_CRYPTO_CONFIG = "$B:CRYPTO_CONFIG";

    /**
     * 缓存 - 第三方接入信息：{rootOrgId}_{appId}
     */
    String CACHE_B_THIRD_PARTY_ACCESS = "$B:THIRD_PARTY_ACCESS:";

    /**
     * 缓存 - 机构拥有的权限信息：{rootOrgId}
     */
    String CACHE_B_ORG_PRIVILEGE = "$B:ORG_PRIVILEGE:";

    /**
     * 缓存 - 角色拥有的权限信息：{rootOrgId}_{roleId}
     */
    String CACHE_B_ROLE_PRIVILEGE = "$B:ROLE_PRIVILEGE:";

    /**
     * 缓存 - 权限关联的角色信息：{rootOrgId}_{privilegeCode}
     */
    String CACHE_B_PRIVILEGE_ROLES = "$B:PRIVILEGE_ROLES:";

    /**
     * 缓存 - 数据权限信息：{userId}_{dataRuleType}
     */
    String CACHE_B_DATA_RULE = "$B:DATA_RULE:";

    /**
     * 缓存 - 系统属性：{propKey}
     */
    String CACHE_B_SYS_PROP = "$B:SYS_PROP:";

    /**
     * 缓存 - 短信配置信息：{smsCode}
     */
    String CACHE_B_SMS_PROP = "$B:SMS_PROP:";

    /**
     * 缓存 - 顶级机构信息：{domain}
     */
    String CACHE_B_ROOT_ORG = "$B:ROOT_ORG:";

    /**
     * 缓存 - 机构信息：{orgId}
     */
    String CACHE_B_ORG = "$B:ORG:";

    /**
     * 缓存 - 机构属性信息：{orgId}_{key}
     */
    String CACHE_B_ORG_PROP = "$B:ORG_PROP:";

    /**
     * 缓存 - 机构属性列表信息：{orgId}_{propertyGroupId}
     */
    String CACHE_B_ORG_PROP_LIST = "$B:ORG_PROP_LIST:";

    /**
     * 缓存 - 课程信息：{courseId}
     */
    String CACHE_B_COURSE = "$B:COURSE:";

    /**
     * 缓存 - 考生信息：{studentId}
     */
    String CACHE_B_STUDENT = "$B:STUDENT:";

    /**
     * 缓存 - 学习中心安全策略IP：{orgId}
     */
    String CACHE_B_SECURITY_IP = "$B:SECURITY_IP:";

    /**
     * 缓存 - 登录错误信息：{accountType}_{account}_{ip}
     */
    String CACHE_B_LOGIN_ERR = "$B:LOGIN_ERR:";

    /**
     * 缓存 - 极验-验证码状态：{user_id}
     */
    String CACHE_B_GEETEST_STATUS = "$B:GEETEST_STATUS:UID_";

    /**
     * 缓存 - 数字公式-验证码信息：{rootOrgId}_{account}
     */
    String CACHE_B_VERIFY_CODE = "$B:VERIFY_CODE:";

    /**
     * 缓存 - 数字公式-验证码限流：{rootOrgId}_{account}
     */
    String CACHE_B_VERIFY_CODE_LIMIT = "$B:VERIFY_CODE_LIMIT:";

    /**
     * 缓存 - 数字公式-验证码资源标识：{uuid}
     */
    String CACHE_B_VERIFY_CODE_RESOURCE = "$B:VERIFY_CODE_RESOURCE:";



    /* #################### 考务模块 #################### */

    /**
     * 缓存 - 考试设置信息：{examId}
     */
    String CACHE_E_EXAM = "$E:EXAM:";

    /**
     * 缓存 - 考试属性信息：{examId}_{key}
     */
    String CACHE_E_EXAM_PROP = "$E:EXAM_PROP:";

    /**
     * 缓存 - 考试场次信息：{examId}_{examStageId}
     */
    String CACHE_E_EXAM_STAGE = "$E:EXAM_STAGE:";

    /**
     * 缓存 - 考试场次集合信息：{examId}
     */
    String CACHE_E_EXAM_STAGES = "$E:EXAM_STAGES:";

    /**
     * 缓存 - 考试机构设置信息：{examId}_{orgId}
     */
    String CACHE_E_EXAM_ORG_SETTINGS = "$E:EXAM_ORG_SETTINGS:";

    /**
     * 缓存 - 考试机构属性信息：{examId}_{orgId}_{key}
     */
    String CACHE_E_EXAM_ORG_PROP = "$E:EXAM_ORG_PROP:";

    /**
     * 缓存 - 考试学生设置信息：{examId}_{studentId}
     */
    String CACHE_E_EXAM_STUDENT_SETTINGS = "$E:EXAM_STUDENT_SETTINGS:";

    /**
     * 缓存 - 考试学生属性信息：{examId}_{studentId}_{key}
     */
    String CACHE_E_EXAM_STUDENT_PROP = "$E:EXAM_STUDENT_PROP:";



    /* #################### 题库模块 #################### */

    /**
     * 缓存 - 试卷结构：{paperStructId}
     */
    String CACHE_Q_PAPER_STRUCT = "$Q:PAPER_STRUCT:";

    /**
     * 缓存 - 调卷规则：{examId}_{courseCode}
     */
    String CACHE_Q_EXTRACT_CONFIG = "$Q:EXTRACT_CONFIG:";

    /**
     * 缓存 - 调卷规则的试卷结构：{examId}_{courseCode}_{groupCode}_{paperId}
     */
    String CACHE_Q_EXTRACT_CONFIG_PAPER = "$Q:EXTRACT_CONFIG_PAPER:";

    /**
     * 缓存 - 随机抽卷模板：{randomPaperId}
     */
    String CACHE_Q_RANDOM_PAPER = "$Q:RANDOM_PAPER:";

    /**
     * 缓存 - 试卷信息：{paperId}
     */
    String CACHE_Q_BASE_PAPER = "$Q:BASE_PAPER:";

    /**
     * 缓存 - 临时封装的试卷信息：{paperId}
     */
    String CACHE_Q_TEMP_PAPER = "$Q:TEMP_PAPER:";

    /**
     * 缓存 - 试题信息：{examId}_{courseCode}_{groupCode}_{questionId}
     */
    String CACHE_Q_QUESTION = "$Q:QUESTION:";

    /**
     * 缓存 - 试题答案信息：{questionId}
     */
    String CACHE_Q_QUESTION_ANSWER = "$Q:QUESTION_ANSWER:";



    /* #################### 网考模块 #################### */

    /**
     * 缓存 - 网考考试会话：{studentId}
     */
    String CACHE_OE_SESSION = "$OE:SESSION:";

    /**
     * 缓存 - 考试次数控制：{examStudentId}
     */
    String CACHE_OE_BOSS = "$OE:BOSS:";

    /**
     * 缓存 - 网考考试记录：{examRecordDataId}
     */
    String CACHE_OE_RECORD = "$OE:RECORD:";

    /**
     * 缓存 - 考试心跳：{examRecordDataId}
     */
    String CACHE_OE_HEARTBEAT = "$OE:HEARTBEAT:";

    /**
     * 缓存 - 考试活动时间：{examRecordDataId}
     */
    String CACHE_OE_ACTIVE = "$OE:ACTIVE:";

    /**
     * 缓存 - 网考试卷结构：{examRecordDataId}
     */
    String CACHE_OE_PAPER = "$OE:PAPER:";

    /**
     * 缓存 - 网考作答：{examRecordDataId}_{order}
     */
    String CACHE_OE_ANSWER = "$OE:ANSWER:";

    /**
     * 缓存 - 网考文件作答：{examRecordDataId}_{order}
     */
    String CACHE_OE_FILE_ANSWER = "$OE:FILE_ANSWER:";

    /**
     * 缓存 - 网考考生信息：{examStudentId}
     */
    String CACHE_OE_EXAM_STUDENT = "$OE:EXAM_STUDENT:";

    /**
     * 缓存 - 考试成绩统计状态：{examId}
     */
    String CACHE_OE_SCORE_STATISTIC_STATUS = "$OE:SCORE_STATISTIC_STATUS:";

    /**
     * 缓存 - 违纪非法考生端应用：{examRecordDataId}
     */
    String CACHE_OE_DISCIPLINE_ILLEGAL_CLIENT = "$OE:DISCIPLINE_ILLEGAL_CLIENT:";

    /**
     * 缓存 - 违纪非法数据：{examRecordDataId}
     */
    String CACHE_OE_DISCIPLINE_ILLEGAL_DATA = "$OE:DISCIPLINE_ILLEGAL_DATA:";

    /**
     * 缓存 - 同步人脸比对结果：{studentId}
     */
    String CACHE_OE_SYNC_FACE_COMPARE_RESULT = "$OE:SYNC_FACE_COMPARE_RESULT:";

    /**
     * 缓存 - 又拍云图片存储签名：{userId}_{signIdentifier}
     */
    String CACHE_OE_UPYUN_IMAGE_SIGN = "$OE:UPYUN_IMAGE_SIGN:";

    /**
     * 缓存 - 人脸验证限流-秒级：{userKey}
     */
    String CACHE_OE_FACE_API_LIMIT_SECOND = "$OE:FACE_API_LIMIT:S_";

    /**
     * 缓存 - 人脸验证限流-分钟级：{userKey}
     */
    String CACHE_OE_FACE_API_LIMIT_MINUTE = "$OE:FACE_API_LIMIT:M_";

    /**
     * 缓存 - 旧版API黑名单：{rootOrgIds}
     */
    String CACHE_OE_OLD_API_BLACK_LIST = "$OE:OLD_API_BLACK_LIST";

    /**
     * 缓存 - 异步导出任务终止：{taskId}
     */
    String CACHE_OE_EXPORT_TASK_STOP = "$OE:EXPORT_TASK_STOP:";



    /* #################### Exchange模块 #################### */

    /**
     * 缓存 - 课程组下课程列表：{rootOrgId}_{courseGroupName}
     */
    String CACHE_EX_COURSE_LIST = "$EX:COURSE_LIST:";

    /**
     * 缓存 - 短信验证码：{phone}
     */
    String CACHE_EX_SMS = "$EX:SMS:";



    /* #################### APP模块 #################### */

    /**
     * 缓存 - 请求Trace：{threadTraceID}
     */
    String CACHE_APP_REQ_TRACE = "$APP:REQ_TRACE:";


}
