package cn.com.qmth.examcloud.starters.crypto.common;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 加、解密相关辅助类
 */
public class CryptoHelper implements CryptoConstant {

    private static final Logger log = LoggerFactory.getLogger(CryptoHelper.class);

    public static long parseTimestamp(String timestamp) {
        if (StringUtils.isEmpty(timestamp)) {
            throw new CryptoException(REQUEST_PARAM_ERROR);
        }

        try {
            return Long.parseLong(timestamp);
        } catch (NumberFormatException e) {
            log.error(e.getMessage());
            throw new CryptoException(REQUEST_PARAM_ERROR);
        }
    }

    public static void verifyTimestamp(long timestamp) {
        long diff = System.currentTimeMillis() - timestamp;
        if (Math.abs(diff) > 60L * 1000L) {
            // 60秒内有效
            throw new CryptoException(REQUEST_PARAM_ERROR);
        }
    }

    public static String buildKey(FieldPair... pairs) {
        StringBuilder params = new StringBuilder();

        for (int n = 0; n < pairs.length; n++) {
            FieldPair pair = pairs[n];
            params.append(pair.getFieldName()).append("=").append(pair.getFieldValue());
            if (n < pairs.length - 1) {
                params.append("&");
            }
        }

        String key = DigestUtils.md5Hex(params.toString());
        log.info("Crypto key = {}, params = {}", key, params);
        return key;
    }

}
