package cn.com.qmth.examcloud.starters.crypto.service.impl;

import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoException;
import cn.com.qmth.examcloud.starters.crypto.common.Key;
import cn.com.qmth.examcloud.starters.crypto.common.KeyType;
import cn.com.qmth.examcloud.starters.crypto.service.CryptoService;
import cn.com.qmth.examcloud.starters.crypto.utils.RsaUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

/**
 * RSA算法
 */
public class RsaCryptoService implements CryptoService {

    private static final Logger log = LoggerFactory.getLogger(RsaCryptoService.class);

    @Override
    public String encrypt(String str, Key key) {
        if (key == null || StringUtils.isEmpty(key.getValue())) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        if (KeyType.PRIVATE == key.getType()) {
            RSAPrivateKey rsaPrivateKey = RsaUtil.getPrivateKey(key.getValue());
            return RsaUtil.privateEncrypt(str, rsaPrivateKey);
        }

        if (KeyType.PUBLIC == key.getType()) {
            RSAPublicKey rsaPublicKey = RsaUtil.getPublicKey(key.getValue());
            return RsaUtil.publicEncrypt(str, rsaPublicKey);
        }

        throw new CryptoException(CryptoConstant.ENCRYPT_ERROR);
    }

    @Override
    public String decrypt(String str, Key key) {
        if (key == null || StringUtils.isEmpty(key.getValue())) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        if (KeyType.PRIVATE == key.getType()) {
            RSAPrivateKey rsaPrivateKey = RsaUtil.getPrivateKey(key.getValue());
            return RsaUtil.privateDecrypt(str, rsaPrivateKey);
        }

        if (KeyType.PUBLIC == key.getType()) {
            RSAPublicKey rsaPublicKey = RsaUtil.getPublicKey(key.getValue());
            return RsaUtil.publicDecrypt(str, rsaPublicKey);
        }

        throw new CryptoException(CryptoConstant.DECRYPT_ERROR);
    }

}
