package cn.com.qmth.examcloud.starters.crypto.service.impl;

import cn.com.qmth.examcloud.starters.crypto.common.CryptoConstant;
import cn.com.qmth.examcloud.starters.crypto.common.CryptoException;
import cn.com.qmth.examcloud.starters.crypto.common.Key;
import cn.com.qmth.examcloud.starters.crypto.service.CryptoService;
import cn.com.qmth.examcloud.starters.crypto.utils.TripleDesUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;

/**
 * TripleDes算法（默认不带向量）
 */
public class TripleDesCryptoService implements CryptoService {

    private static final Logger log = LoggerFactory.getLogger(TripleDesCryptoService.class);

    @Override
    public String encrypt(String str, Key key) {
        if (key == null || StringUtils.isEmpty(key.getValue())) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        byte[] keyBytes = key.getValue().getBytes(StandardCharsets.UTF_8);
        return TripleDesUtil.encrypt(str, keyBytes, false);
    }

    @Override
    public String decrypt(String str, Key key) {
        if (key == null || StringUtils.isEmpty(key.getValue())) {
            throw new CryptoException(CryptoConstant.KEY_ERROR);
        }

        byte[] keyBytes = key.getValue().getBytes(StandardCharsets.UTF_8);
        return TripleDesUtil.decrypt(str, keyBytes, false);
    }

}
