package cn.com.qmth.examcloud.starters.face.verify.common;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.text.SimpleDateFormat;

public class JsonHelper {

    private static Logger log = LoggerFactory.getLogger(JsonHelper.class);

    private ObjectMapper mapper;

    public JsonHelper() {
        mapper = new ObjectMapper();

        //设置输入时忽略在JSON字符串中存在但Java对象实际没有的属性
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);

        //忽略无法转换的对象
        mapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);

        //设置默认日期格式
        mapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
    }

    public JsonHelper style(Include include) {
        if (include != null) {
            mapper.setSerializationInclusion(include);
        }
        return this;
    }

    public String toJson(Object object) {
        if (object == null) {
            return null;
        }

        try {
            return mapper.writeValueAsString(object);
        } catch (IOException e) {
            log.error("toJson error!" + e.getMessage(), e);
            return null;
        }
    }

    public <T> T parseJson(String jsonStr, Class<T> clazz) {
        if (StringUtils.isEmpty(jsonStr)) {
            return null;
        }

        try {
            return mapper.readValue(jsonStr, clazz);
        } catch (IOException e) {
            log.error("parseJson error!" + e.getMessage(), e);
            return null;
        }
    }

}
