package cn.com.qmth.examcloud.commons.helpers.poi;

import org.apache.commons.compress.utils.IOUtils;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.xssf.eventusermodel.XSSFReader;
import org.apache.poi.xssf.model.SharedStringsTable;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.InputStream;
import java.util.Iterator;

/**
 * excel xlsx 文件解析
 *
 * @author WANGWEI
 * @date 2018年8月1日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public abstract class XlsxHandler extends DefaultHandler {

    private String[] row;

    private SharedStringsTable sst;

    private String cellValue;

    private boolean isString;

    private int sheetIndex = -1;

    private int curRow = 0;

    private int curColumnIndex = 0;

    private int columnSize = 0;

    /**
     * 构造函数
     *
     * @param columnSize
     */
    public XlsxHandler(int columnSize) {
        super();
        this.columnSize = columnSize;
    }

    /**
     * 方法注释
     *
     * @param sheetIndex
     * @param curRow
     * @param row
     * @author WANGWEI
     */
    public abstract void optRows(int sheetIndex, int curRow, String[] row);

    /**
     * 方法注释
     *
     * @param filePath
     * @param sheetId
     * @throws Exception
     * @author WANGWEI
     */
    public void processOneSheet(String filePath, int sheetId) throws Exception {
        OPCPackage pkg = null;
        InputStream is = null;
        try {
            pkg = OPCPackage.open(filePath);
            XSSFReader r = new XSSFReader(pkg);
            SharedStringsTable sst = r.getSharedStringsTable();

            XMLReader parser = XMLReaderFactory.createXMLReader();
            this.sst = sst;
            parser.setContentHandler(this);
            is = r.getSheet("rId" + sheetId);
            sheetIndex++;
            InputSource sheetSource = new InputSource(is);
            parser.parse(sheetSource);
        } finally {
            IOUtils.closeQuietly(is);
            IOUtils.closeQuietly(pkg);
        }
    }

    public void processOneSheet(InputStream is, int sheetId) throws Exception {
        OPCPackage pkg = null;
        try {
            pkg = OPCPackage.open(is);
            XSSFReader r = new XSSFReader(pkg);
            SharedStringsTable sst = r.getSharedStringsTable();

            XMLReader parser = XMLReaderFactory.createXMLReader();
            this.sst = sst;
            parser.setContentHandler(this);
            is = r.getSheet("rId" + sheetId);
            sheetIndex++;
            InputSource sheetSource = new InputSource(is);
            parser.parse(sheetSource);
        } finally {
            IOUtils.closeQuietly(is);
            IOUtils.closeQuietly(pkg);
        }
    }

    /**
     * 方法注释
     *
     * @param filePath
     * @throws Exception
     * @author WANGWEI
     */
    public void process(String filePath) throws Exception {
        OPCPackage pkg = null;
        try {
            pkg = OPCPackage.open(filePath);
            XSSFReader r = new XSSFReader(pkg);
            SharedStringsTable sst = r.getSharedStringsTable();

            XMLReader parser = XMLReaderFactory.createXMLReader();
            this.sst = sst;
            parser.setContentHandler(this);

            Iterator<InputStream> sheets = r.getSheetsData();
            while (sheets.hasNext()) {
                curRow = 0;
                sheetIndex++;
                InputStream sheet = null;
                try {
                    sheet = sheets.next();
                    InputSource sheetSource = new InputSource(sheet);
                    parser.parse(sheetSource);
                } finally {
                    IOUtils.closeQuietly(sheet);
                }
            }
        } finally {
            IOUtils.closeQuietly(pkg);
        }
    }

    @Override
    public void startElement(String uri, String localName, String name, Attributes attributes)
            throws SAXException {
        if (name.equals("c")) {
            String cellType = attributes.getValue("t");
            String x = attributes.getValue("r");
            curColumnIndex = this.getColunmIndex(x);
            if (cellType != null && cellType.equals("s")) {
                isString = true;
            } else {
                isString = false;
            }
        }
        cellValue = "";
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        cellValue += new String(ch, start, length);
    }

    @Override
    public void endElement(String uri, String localName, String name) throws SAXException {
        if (isString) {
            try {
                int idx = Integer.parseInt(cellValue);
                cellValue = new XSSFRichTextString(sst.getEntryAt(idx)).toString();
            } catch (Exception e) {

            }
        }

        if (null == row) {
            row = new String[columnSize];
        }

        if (name.equals("v")) {
            String value = cellValue.trim();
            if (curColumnIndex - 1 < columnSize) {
                row[curColumnIndex - 1] = value;
            }
        } else if (name.equals("row")) {
            optRows(sheetIndex, curRow, row);
            row = null;
            curRow++;
            curColumnIndex = 0;
        }

    }

    /**
     * 方法注释
     *
     * @param x
     * @return
     * @author WANGWEI
     */
    private int getColunmIndex(String x) {
        x = x.replaceAll("[^A-Z]", "");
        byte[] bytes = x.getBytes();
        int len = bytes.length;
        float num = 0;
        for (int i = 0; i < len; i++) {
            num += (bytes[i] - 'A' + 1) * Math.pow(26, len - i - 1);
        }
        return (int) num;
    }

    public int getColumnSize() {
        return columnSize;
    }

    public void setColumnSize(int columnSize) {
        this.columnSize = columnSize;
    }

}