package cn.com.qmth.examcloud.commons.util;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import java.io.*;

/**
 * IO
 *
 * @author WANGWEI
 * @date 2018年6月29日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
public class IOUtil {

    /**
     * 文件转字节数组
     *
     * @param file
     * @return
     * @author WANGWEI
     */
    public static byte[] toByteArray(File file) {
        FileInputStream is = null;
        try {
            is = new FileInputStream(file);
            byte[] byteArray = IOUtils.toByteArray(is);
            return byteArray;
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            IOUtils.closeQuietly(is);
        }
    }

    /**
     * 字节数组转文件
     *
     * @param bfile
     * @param filePath
     * @author WANGWEI
     */
    public static void toFile(byte[] bfile, String filePath) {
        BufferedOutputStream bos = null;
        FileOutputStream fos = null;
        try {
            File file = new File(filePath);
            FileUtils.forceMkdir(file.getParentFile());
            fos = new FileOutputStream(file);
            bos = new BufferedOutputStream(fos);
            bos.write(bfile);
        } catch (Exception e) {
            throw new RuntimeException("", e);
        } finally {
            IOUtils.closeQuietly(bos);
            IOUtils.closeQuietly(fos);
        }
    }

    /**
     * 输入流转换为限制大小的字节
     *
     * @param in
     * @param maxByteSize
     * @return
     * @author WANGWEI
     */
    public static byte[] toLimitedByteArray(InputStream in, long maxByteSize) {
        ByteArrayOutputStream out = null;
        int bufferSize = 1024 * 4;
        int totalSize = 0;
        byte[] buffer = new byte[bufferSize];
        try {
            out = new ByteArrayOutputStream();
            int n = 0;
            while ((n = in.read(buffer)) != -1) {
                out.write(buffer, 0, n);
                totalSize += n;

                if (maxByteSize < totalSize) {
                    throw new RuntimeException("文件大小超过" + (maxByteSize / 1024) + "KB限制");
                }
            }
            return out.toByteArray();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (RuntimeException e) {
            throw e;
        } finally {
            IOUtils.closeQuietly(out);
            IOUtils.closeQuietly(in);
        }
    }

}