package cn.com.qmth.examcloud.support.handler;

import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestion;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionOption;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.support.handler.richtext2.RichTextConverter;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

public class QuestionBodyHandler {

    public static void convertRichText(DefaultQuestion defaultQuestion) {
        if (defaultQuestion == null) {
            return;
        }

        DefaultQuestionStructure structure = defaultQuestion.getMasterVersion();
        if (structure == null) {
            return;
        }

        JsonMapper jsonMapper = JsonMapper.nonNullMapper();

        if (StringUtils.isNotEmpty(structure.getBody())) {
            structure.setBody(jsonMapper.toJson(RichTextConverter.parse(structure.getBody())));
        }

        List<DefaultQuestionUnit> questionUnits = structure.getQuestionUnitList();
        if (CollectionUtils.isEmpty(questionUnits)) {
            return;
        }

        for (DefaultQuestionUnit questionUnit : questionUnits) {
            // 处理题干
            questionUnit.setBody(jsonMapper.toJson(RichTextConverter.parse(questionUnit.getBody())));

            List<DefaultQuestionOption> questionOptions = questionUnit.getQuestionOptionList();
            if (CollectionUtils.isNotEmpty(questionOptions)) {
                for (DefaultQuestionOption questionOption : questionOptions) {
                    // 处理选项
                    questionOption.setBody(jsonMapper.toJson(RichTextConverter.parse(questionOption.getBody())));
                }
            }

            String[] rightAnswers = questionUnit.getRightAnswer();
            if (ArrayUtils.isNotEmpty(rightAnswers)) {
                for (int n = 0; n < rightAnswers.length; n++) {
                    // 处理正确答案
                    rightAnswers[n] = jsonMapper.toJson(RichTextConverter.parse(rightAnswers[n]));
                }
            }
        }
    }

}
