package cn.com.qmth.examcloud.support.privilege;

import cn.com.qmth.examcloud.api.commons.security.bean.Role;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.PrivilegeRolesCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.RootOrgPrivilegesCacheBean;

import java.util.List;
import java.util.Set;

/**
 * 权限管理器
 *
 * @author WANGWEI
 * @date 2019年7月10日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class PrivilegeManager {

    /**
     * 判断角色是否有权限
     *
     * @param rootOrgId
     * @param roleList
     * @param privilegeCode
     * @return
     * @author WANGWEI
     */
    public static Boolean judge(Long rootOrgId, List<Role> roleList, String privilegeCode) {

        PrivilegeRolesCacheBean cacheBean = CacheHelper.getPrivilegeRoles(rootOrgId, privilegeCode);
        Set<Long> roleIdList = cacheBean.getRoleIdList();
        Set<String> roleCodeList = cacheBean.getRoleCodeList();

        for (Role cur : roleList) {
            Long roleId = cur.getRoleId();
            String roleCode = cur.getRoleCode();

            if (null != roleId && roleIdList.contains(roleId)) {
                return true;
            }
            if (null != roleCode && roleCodeList.contains(roleCode)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断顶级机构是否有权限
     *
     * @param rootOrgId
     * @param privilegeCode
     * @return
     * @author WANGWEI
     */
    public static Boolean judge(Long rootOrgId, String privilegeCode) {

        RootOrgPrivilegesCacheBean cacheBean = CacheHelper.getRootOrgPrivileges(rootOrgId);

        Set<String> privilegeCodeList = cacheBean.getPrivilegeCodeList();
        if (privilegeCodeList.contains(privilegeCode)) {
            return true;
        }

        return false;
    }

}
