package cn.com.qmth.examcloud.support.redis;

import org.apache.commons.lang3.StringUtils;
import org.springframework.cglib.proxy.Enhancer;
import org.springframework.cglib.proxy.MethodInterceptor;
import org.springframework.cglib.proxy.MethodProxy;

import java.lang.reflect.Method;

/**
 * redis key helper
 *
 * @author WANGWEI
 * @date 2019年12月12日
 */
public class RedisKeyHelper {

    private static RedisKeyBuilder redisKeyBuilder;

    private static void newInstance() {
        Enhancer enhancer = new Enhancer();
        enhancer.setSuperclass(RedisKeyBuilder.class);
        enhancer.setCallback(new MethodInterceptor() {
            @Override
            public Object intercept(Object obj, Method method, Object[] objects, MethodProxy methodProxy)
                    throws Throwable {
                RedisKeyDefine define = method.getAnnotation(RedisKeyDefine.class);
                String prefix = define.Prefix();
                return prefix + StringUtils.join(objects, '_');
            }
        });

        redisKeyBuilder = (RedisKeyBuilder) enhancer.create();
    }

    public static RedisKeyBuilder getBuilder() {
        if (null == redisKeyBuilder) {
            synchronized (RedisKeyHelper.class) {
                if (null == redisKeyBuilder) {
                    newInstance();
                }
            }
        }

        return redisKeyBuilder;
    }

}
