package cn.com.qmth.examcloud.web.actuator;

import cn.com.qmth.examcloud.commons.util.Calculator;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.support.ApiInfo;
import cn.com.qmth.examcloud.web.support.ApiInfoHolder;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

@Component
public class ApiStatusInfoCollector {

    private static String algorithm;

    {
        // 耗时直方图算法:U:uniform ; B:biased
        algorithm = PropertyHolder.getString("$API.statistic.histograms.algorithm", "B");
        if (!(algorithm.equals("B") || algorithm.equals("U"))) {
            throw new RuntimeException("histograms algorithm must be 'U' or 'B'");
        }
    }

    public synchronized List<ApiStatusInfo> collect(ReportInfo reportInfo) {

        List<HistogramInfo> histogramInfoList = reportInfo.getHistogramInfoList();
        List<MeterInfo> meterInfoList = reportInfo.getMeterInfoList();
        List<TimerInfo> timerInfoList = reportInfo.getTimerInfoList();

        Map<String, MeterInfo> apiErrMeterInfoMap = Maps.newHashMap();
        for (MeterInfo meterInfo : meterInfoList) {
            String key = meterInfo.getKey();
            if (key.startsWith(MetricNames.API_ERROR_METER.name())) {
                String mapping = key.substring(MetricNames.API_ERROR_METER.name().length() + 1);
                apiErrMeterInfoMap.put(mapping, meterInfo);
            }
        }

        Map<String, HistogramInfo> apiHistogramInfoMap = Maps.newHashMap();
        if (algorithm.equals("U")) {
            for (HistogramInfo histogramInfo : histogramInfoList) {
                String key = histogramInfo.getKey();
                if (key.startsWith(MetricNames.API_HISTOGRAM.name())) {
                    String mapping = key.substring(MetricNames.API_HISTOGRAM.name().length() + 1);
                    apiHistogramInfoMap.put(mapping, histogramInfo);
                }
            }
        }

        Map<String, TimerInfo> apiTimerInfoMap = Maps.newHashMap();
        if (algorithm.equals("B")) {
            for (TimerInfo timerInfo : timerInfoList) {
                String key = timerInfo.getKey();
                if (key.startsWith(MetricNames.API_TIMER.name())) {
                    String mapping = key.substring(MetricNames.API_TIMER.name().length() + 1);
                    apiTimerInfoMap.put(mapping, timerInfo);
                }
            }
        }

        List<ApiStatusInfo> list = Lists.newArrayList();

        for (MeterInfo cur : meterInfoList) {
            String key = cur.getKey();
            if (key.startsWith(MetricNames.API_METER.name())) {

                String mapping = key.substring(MetricNames.API_METER.name().length() + 1);

                ApiStatusInfo apiStatusInfo = new ApiStatusInfo();
                list.add(apiStatusInfo);

                apiStatusInfo.setMapping(mapping);

                ApiInfo apiInfo = ApiInfoHolder.getApiInfo(mapping);
                if (null != apiInfo) {
                    apiStatusInfo.setDescription(apiInfo.getDescription());
                }

                apiStatusInfo.setCount(cur.getCount());
                apiStatusInfo.setMeanRate(cur.getMeanRate());
                apiStatusInfo.setOneMinuteRate(cur.getOneMinuteRate());
                apiStatusInfo.setFiveMinuteRate(cur.getFiveMinuteRate());
                apiStatusInfo.setFifteenMinuteRate(cur.getFifteenMinuteRate());

                if (algorithm.equals("U")) {
                    HistogramInfo histogramInfo = apiHistogramInfoMap.get(mapping);

                    apiStatusInfo.setMin(histogramInfo.getMin());
                    apiStatusInfo.setMax(histogramInfo.getMax());
                    apiStatusInfo.setMean(histogramInfo.getMean());

                    apiStatusInfo.setP50(histogramInfo.getP50());
                    apiStatusInfo.setP75(histogramInfo.getP75());
                    apiStatusInfo.setP95(histogramInfo.getP95());
                    apiStatusInfo.setP98(histogramInfo.getP98());
                    apiStatusInfo.setP99(histogramInfo.getP99());
                }

                if (algorithm.equals("B")) {
                    TimerInfo timerInfo = apiTimerInfoMap.get(mapping);

                    apiStatusInfo.setMin(timerInfo.getMin());
                    apiStatusInfo.setMax(timerInfo.getMax());
                    apiStatusInfo.setMean(timerInfo.getMean());

                    apiStatusInfo.setP50(timerInfo.getP50());
                    apiStatusInfo.setP75(timerInfo.getP75());
                    apiStatusInfo.setP95(timerInfo.getP95());
                    apiStatusInfo.setP98(timerInfo.getP98());
                    apiStatusInfo.setP99(timerInfo.getP99());
                }

                MeterInfo apiErrMeterInfo = apiErrMeterInfoMap.get(mapping);
                if (null == apiErrMeterInfo) {
                    apiStatusInfo.setErrorCount(0L);
                    apiStatusInfo.setErrorMeanRate(0L);
                    apiStatusInfo.setErrorPercent(0D);
                } else {
                    apiStatusInfo.setErrorCount(apiErrMeterInfo.getCount());
                    apiStatusInfo.setErrorMeanRate(apiErrMeterInfo.getMeanRate());
                    apiStatusInfo.setErrorPercent(
                            getPercent(apiErrMeterInfo.getCount(), cur.getCount()));
                }

            }
        }

        return list;
    }

    private Double getPercent(double v1, double v2) {
        if (0 == v1 || 0 == v2) {
            return 0d;
        }
        double ret = Calculator.divide(v1, v2, 3);
        return ret;
    }

}
