package cn.com.qmth.examcloud.web.cache;

import cn.com.qmth.examcloud.api.commons.CloudService;
import cn.com.qmth.examcloud.api.commons.enums.BasicDataType;
import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * cache
 *
 * @author WANGWEI
 * @date 2018年8月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("cache")
public class CacheCloudServiceProvider implements CloudService {

    private static final long serialVersionUID = -5326807830421467943L;

    private static final Logger LOG = LoggerFactory.getLogger(CacheCloudServiceProvider.class);

    private static Map<String, ObjectCache<?>> map = Maps.newConcurrentMap();

    private static Map<String, HashCache<?>> hashCacheMap = Maps.newConcurrentMap();

    @PostMapping("refresh")
    public String refresh(@RequestBody RefreshCacheReq req) {

        String className = req.getClassName();
        String[] keys = req.getKeys();
        BasicDataType[] typeArray = req.getTypeArray();

        Object[] expectedKeys = new Object[keys.length];

        for (int i = 0; i < keys.length; i++) {
            String key = keys[i];
            BasicDataType type = typeArray[i];
            if (type.equals(BasicDataType.LONG)) {
                expectedKeys[i] = Long.parseLong(key);
            } else if (type.equals(BasicDataType.STRING)) {
                expectedKeys[i] = key;
            } else if (type.equals(BasicDataType.INTEGER)) {
                expectedKeys[i] = Integer.parseInt(key);
            } else {
                throw new ExamCloudRuntimeException("key type is not supported");
            }
        }

        ObjectCache<?> objectCache = map.get(className);
        if (null == objectCache) {

            try {
                Class<?> c = Class.forName(className);
                objectCache = (ObjectCache<?>) SpringContextHolder.getBean(c);
                map.put(className, objectCache);
            } catch (ClassNotFoundException e) {
                throw new StatusException("008001", "class not found", e);
            }
        }
        objectCache.refresh(expectedKeys);
        Object object = objectCache.get(expectedKeys);
        return JsonUtil.toJson(object);
    }

    @PostMapping("refreshHash")
    public String refreshHash(@RequestBody RefreshHashCacheReq req) {

        String className = req.getClassName();
        String[] keys = req.getKeys();
        String[] subkeys = req.getSubKeys();
        BasicDataType[] typeArray = req.getTypeArray();
        BasicDataType[] subTypeArray = req.getSubTypeArray();

        Object[] expectedKeys = new Object[keys.length];

        for (int i = 0; i < keys.length; i++) {
            String key = keys[i];
            BasicDataType type = typeArray[i];
            if (type.equals(BasicDataType.LONG)) {
                expectedKeys[i] = Long.parseLong(key);
            } else if (type.equals(BasicDataType.STRING)) {
                expectedKeys[i] = key;
            } else if (type.equals(BasicDataType.INTEGER)) {
                expectedKeys[i] = Integer.parseInt(key);
            } else {
                throw new ExamCloudRuntimeException("key type is not supported");
            }
        }

        Object[] expectedSubKeys = new Object[subkeys.length];
        for (int i = 0; i < subkeys.length; i++) {
            String key = subkeys[i];
            BasicDataType type = subTypeArray[i];
            if (type.equals(BasicDataType.LONG)) {
                expectedSubKeys[i] = Long.parseLong(key);
            } else if (type.equals(BasicDataType.STRING)) {
                expectedSubKeys[i] = key;
            } else if (type.equals(BasicDataType.INTEGER)) {
                expectedSubKeys[i] = Integer.parseInt(key);
            } else {
                throw new ExamCloudRuntimeException("subkey type is not supported");
            }
        }

        HashCache<?> hashCache = hashCacheMap.get(className);
        if (null == hashCache) {

            try {
                Class<?> c = Class.forName(className);
                hashCache = (HashCache<?>) SpringContextHolder.getBean(c);
                hashCacheMap.put(className, hashCache);
            } catch (ClassNotFoundException e) {
                throw new StatusException("008002", "class not found", e);
            }
        }
        hashCache.refresh(expectedKeys, expectedSubKeys);
        Object object = hashCache.get(expectedKeys, expectedSubKeys);
        return JsonUtil.toJson(object);
    }

}
