package cn.com.qmth.examcloud.web.cache;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Arrays;

/**
 * Hash redis缓存
 *
 * @param <T>
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public abstract class HashRedisCache<T extends RandomCacheBean> implements HashCache<T> {

    private RedisClient redisClient;

    private RedisClient getRedisClient() {
        if (null == redisClient) {
            redisClient = SpringContextHolder.getBean(RedisClient.class);
        }
        return redisClient;
    }

    protected abstract String getKeyPrefix();

    protected abstract int getTimeout();

    protected String buildKey(Object... keys) {
        String key = getKeyPrefix() + StringUtils.join(Arrays.asList(keys), '_');
        return key;
    }

    protected String buildSubKey(Object... keys) {
        String key = StringUtils.join(Arrays.asList(keys), '_');
        return key;
    }

    private T getFromCache(String key, String subkey) {
        Object object = getRedisClient().get(key, subkey, Object.class);
        @SuppressWarnings("unchecked")
        T t = (T) object;
        return t;
    }

    @Override
    public T get(Object[] keys, Object[] subkeys) {
        String key = buildKey(keys);
        String subkey = buildSubKey(subkeys);
        T t = getFromCache(key, subkey);

        if (null == t) {
            refresh(keys, subkeys);
        }
        t = getFromCache(key, subkey);
        return t;
    }

    @Override
    public void remove(Object... keys) {
        String key = buildKey(keys);
        getRedisClient().delete(key);
    }

    @Override
    public void refresh(Object[] keys, Object[] subkeys) {
        String key = buildKey(keys);
        String subkey = buildSubKey(subkeys);
        T t = null;
        try {
            t = loadFromResource(keys, subkeys);
        } catch (StatusException e) {
            throw e;
        } catch (Exception e) {
            throw new ExamCloudRuntimeException("fail to load data. key=" + key + " subkey=" + subkey, e);
        }
        if (null != t) {
            int timeout = getTimeout();
            if (timeout < 60) {
                timeout = 60;
            }
            getRedisClient().set(key, subkey, t, timeout);
        } else {
            getRedisClient().delete(key, subkey);
        }
    }

}
