package cn.com.qmth.examcloud.web.cache;

import cn.com.qmth.examcloud.api.commons.enums.BasicDataType;
import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.web.cloud.CloudClientSupport;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * redis缓存触发器
 *
 * @author
 * @date 2019年5月9日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component
public class HashRedisCacheTrigger extends CloudClientSupport {

    private static Map<String, Boolean> hasClassMap = Maps.newConcurrentMap();

    private static Map<String, HashCache<?>> hashCacheMap = Maps.newConcurrentMap();

    /**
     * 开火
     *
     * @param appName
     * @param keys
     * @return
     * @author
     */
    public void fire(String appName, String className, Object[] keys, Object[] subkeys) {

        Boolean has = hasClassMap.get(className);
        HashCache<?> hashCache = null;

        if (null != has) {
            if (has) {
                hashCache = hashCacheMap.get(className);
                hashCache.refresh(keys, subkeys);
                return;
            }
        } else {
            try {
                Class<?> c = Class.forName(className);
                hashCache = (HashCache<?>) SpringContextHolder.getBean(c);
                hashCache.refresh(keys, subkeys);
                hashCacheMap.put(className, hashCache);
                hasClassMap.put(className, true);
                return;
            } catch (ClassNotFoundException e) {
                hasClassMap.put(className, false);
            }
        }

        String[] keyArray = new String[keys.length];
        BasicDataType[] typeArray = new BasicDataType[keys.length];

        for (int i = 0; i < keys.length; i++) {
            keyArray[i] = String.valueOf(keys[i]);
            Class<? extends Object> c = keys[i].getClass();
            if (c.equals(Long.class)) {
                typeArray[i] = BasicDataType.LONG;
            } else if (c.equals(String.class)) {
                typeArray[i] = BasicDataType.STRING;
            } else if (c.equals(Integer.class)) {
                typeArray[i] = BasicDataType.INTEGER;
            } else {
                throw new ExamCloudRuntimeException("key type is not supported");
            }
        }

        String[] subKeyArray = new String[subkeys.length];
        BasicDataType[] subTypeArray = new BasicDataType[subkeys.length];

        for (int i = 0; i < subkeys.length; i++) {
            subKeyArray[i] = String.valueOf(subkeys[i]);
            Class<? extends Object> c = subkeys[i].getClass();
            if (c.equals(Long.class)) {
                subTypeArray[i] = BasicDataType.LONG;
            } else if (c.equals(String.class)) {
                subTypeArray[i] = BasicDataType.STRING;
            } else if (c.equals(Integer.class)) {
                subTypeArray[i] = BasicDataType.INTEGER;
            } else {
                throw new ExamCloudRuntimeException("subkey type is not supported");
            }
        }

        RefreshHashCacheReq req = new RefreshHashCacheReq();
        req.setClassName(className);
        req.setKeys(keyArray);
        req.setTypeArray(typeArray);
        req.setSubKeys(subKeyArray);
        req.setSubTypeArray(subTypeArray);
        post(appName, "refreshHash", req);

    }

    @Override
    protected String getRequestMappingPrefix() {
        return "cache";
    }

}
