package cn.com.qmth.examcloud.web.config;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.File;

/**
 * 系统配置
 *
 * @author WANGWEI
 * @date 2018年9月7日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component
public class SystemProperties {

    private static final Logger log = LoggerFactory.getLogger(SystemProperties.class);

    /**
     * 配置文件环境
     */
    @Value("${spring.profiles.active:}")
    private String profile;

    /**
     * 文件存放目录
     */
    @Value("${examcloud.web.sys.dataDir:dataDir}")
    private String dataDir;

    /**
     * 文件存放临时目录
     */
    @Value("${examcloud.web.sys.tempDataDir:tempDataDir}")
    private String tempDataDir;

    @Value("${$aliyun.site.1.bucket}")
    private String aliyunBucketName;

    /**
     * 正式环境，此地址默认为阿里云内网域名（阿里云环境时，推荐使用）
     * 其它环境：非阿里云内网域名
     */
    @Value("${$aliyun.site.1.ossEndpoint}")
    private String aliyunOssEndpoint;

    @Value("${$aliyun.site.1.domain}")
    private String aliyunDomain;

    @Value("${$aliyun.site.1.accessKeyId}")
    private String accessKeyId;

    @Value("${$aliyun.site.1.accessKeySecret}")
    private String accessKeySecret;

    /**
     * 应用ID
     */
    public static Integer APP_ID;

    /**
     * 应用密钥
     */
    public static String APP_SECRET_KEY;

    /**
     * 应用编码
     */
    public static String APP_CODE;

    @Value("${examcloud.bootstrap.app.appId:-1}")
    public void appId(Integer appId) {
        APP_ID = appId;
    }

    @Value("${examcloud.bootstrap.app.secretKey:}")
    public void appSecretKey(String appSecretKey) {
        APP_SECRET_KEY = appSecretKey;
    }

    /* @Value("${examcloud.startup.appCode:}") */
    @Value("${sys.config.center.appCode:}")
    public void appCode(String appCode) {
        APP_CODE = appCode;
    }

    public String getProfile() {
        return profile;
    }

    public void setProfile(String profile) {
        this.profile = profile;
    }

    public String getDataDir() {
        return dataDir;
    }

    public void setDataDir(String dataDir) {
        this.dataDir = dataDir;
    }

    public String getTempDataDir() {
        return tempDataDir;
    }

    public void setTempDataDir(String tempDataDir) {
        this.tempDataDir = tempDataDir;
    }

    public String getAliyunBucketName() {
        return aliyunBucketName;
    }

    public void setAliyunBucketName(String aliyunBucketName) {
        this.aliyunBucketName = aliyunBucketName;
    }

    public String getAliyunOssEndpoint() {
        return aliyunOssEndpoint;
    }

    public void setAliyunOssEndpoint(String aliyunOssEndpoint) {
        this.aliyunOssEndpoint = aliyunOssEndpoint;
    }

    public String getAliyunDomain() {
        return aliyunDomain;
    }

    public void setAliyunDomain(String aliyunDomain) {
        this.aliyunDomain = aliyunDomain;
    }

    public String getAccessKeyId() {
        return accessKeyId;
    }

    public void setAccessKeyId(String accessKeyId) {
        this.accessKeyId = accessKeyId;
    }

    public String getAccessKeySecret() {
        return accessKeySecret;
    }

    public void setAccessKeySecret(String accessKeySecret) {
        this.accessKeySecret = accessKeySecret;
    }

    /**
     * 将文件存储环境的 公网域名 替换为 阿里云内网域名
     */
    public String replaceFileUrlInternalDomain(String fileUrl) {
        if (fileUrl == null || !fileUrl.contains(getAliyunDomain())) {
            return fileUrl;
        }

        String[] values = getAliyunOssEndpoint().split("\\/\\/");
        String newDomain = values[0] + "//" + getAliyunBucketName() + "." + values[1];

        fileUrl = fileUrl.replace(getAliyunDomain(), newDomain);
        return fileUrl;
    }

    @PostConstruct
    public void init() {
        if (StringUtils.isNotEmpty(dataDir)) {
            File dir = new File(dataDir);
            if (!dir.exists()) {
                dir.mkdirs();
            }
            log.info(dataDir + " init...");
        }

        if (StringUtils.isNotEmpty(tempDataDir)) {
            File dir = new File(tempDataDir);
            if (!dir.exists()) {
                dir.mkdirs();
            }
            log.info(tempDataDir + " init...");
        }
    }

}
