package cn.com.qmth.examcloud.web.filestorage;

import java.io.File;
import java.io.InputStream;

public interface FileStorage {

    /**
     * 保存文件到存储服务
     *
     * @param siteId
     * @param env
     * @param in         文件流
     * @param md5        文件MD5 可为空
     * @param refreshCDN 是否刷新CDN缓存
     * @return 返回路径
     */
    YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, InputStream in, String md5, boolean refreshCDN);

    /**
     * 保存文件到存储服务
     *
     * @param siteId
     * @param env
     * @param file       文件
     * @param md5        文件MD5 可为空
     * @param refreshCDN 是否刷新CDN缓存
     * @return 返回路径
     */
    YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, File file, String md5, boolean refreshCDN);

    YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, File file, String md5, boolean refreshCDN, Long cacheAge);

    /**
     * 保存文件到存储服务
     *
     * @param siteId
     * @param env
     * @param bytes
     * @param refreshCDN 是否刷新CDN缓存
     * @return
     */
    YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, byte[] bytes, boolean refreshCDN);

    /**
     * 获取可直接访问的文件地址
     *
     * @param path 全路径，包含根目录，含协议名
     * @return 返回可直接访问的地址
     */
    String realPath(String path);

    /**
     * 获取可直接访问的文件地址(备用域名地址)
     *
     * @param path 全路径，包含根目录，含协议名
     * @return 返回可直接访问的地址, 如果没有配置备用地址，则返回主域名地址
     */
    String realPathBackup(String path);

    /**
     * 保存文件到存储服务,siteId为1,转换接口用
     *
     * @param file 文件
     * @param path 全路径，含协议名
     * @return 返回路径
     */
    YunPathInfo saveFile(File file, String path);

    /**
     * 获取云存储签名
     *
     * @param siteId
     * @param env
     * @param md5
     * @return
     */
    YunHttpRequest getSignature(String siteId, FileStoragePathEnvInfo env, String md5);

    /**
     * 删除文件
     *
     * @param path 全路径，含协议名
     */
    void deleteFile(String path);

}
