package cn.com.qmth.examcloud.web.helpers;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.Util;
import org.springframework.data.repository.CrudRepository;

import java.util.Arrays;
import java.util.Optional;

/**
 * 全局 helper
 *
 * @author WANGWEI
 * @date 2019年3月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class GlobalHelper {

    /**
     * 顶级机构一致性校验
     *
     * @param rootOrgIds
     * @author WANGWEI
     */
    public static void uniformRootOrg(Long... rootOrgIds) {
        if (!Util.equals(Arrays.asList(rootOrgIds))) {
            throw new StatusException("120", "非法请求(顶级机构不一致)");
        }
    }

    /**
     * 获取存在的实体
     *
     * @param repo
     * @param id
     * @param se
     * @return
     * @author WANGWEI
     */
    public static <ID, T> T getPresentEntity(CrudRepository<T, ID> repo, ID id, Class<T> c) {
        Optional<T> optional = repo.findById(id);
        if (!optional.isPresent()) {
            String simpleName = c.getSimpleName();
            String desc = String.format("%s (ID: %s) is not present", simpleName,
                    JsonUtil.toJson(id));
            throw new StatusException("122", desc);
        } else {
            return optional.get();
        }
    }

    /**
     * 获取实体(不存在返回null)
     *
     * @param repo
     * @param id
     * @param c
     * @return
     * @author WANGWEI
     */
    public static <ID, T> T getEntity(CrudRepository<T, ID> repo, ID id, Class<T> c) {
        Optional<T> optional = repo.findById(id);
        if (!optional.isPresent()) {
            return null;
        } else {
            return optional.get();
        }
    }

}
