package cn.com.qmth.examcloud.web.redis;

import java.util.concurrent.TimeUnit;

/**
 * redis client
 *
 * @author WANGWEI
 * @date 2019年2月22日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public interface RedisClient {

    /**
     * 是否可用
     *
     * @return
     * @author WANGWEI
     */
    public boolean isEnable();

    /**
     * 设置是否可用
     *
     * @param enable
     * @author WANGWEI
     */
    public void setEnable(boolean enable);

    /**
     * 方法注释
     *
     * @param key
     * @param value
     * @author WANGWEI
     */
    public void set(String key, Object value);

    /**
     * @param key
     * @param subkey
     * @param value
     */
    public void set(String key, String subkey, Object value);

    /**
     * 方法注释
     *
     * @param key
     * @param value
     * @param timeout
     * @author WANGWEI
     */
    public void set(String key, Object value, int timeout);

    /**
     * @param key
     * @param subkey
     * @param value
     * @param timeout
     */
    public void set(String key, String subkey, Object value, int timeout);

    /**
     * 方法注释
     *
     * @param key
     * @param timeout
     * @author WANGWEI
     */
    public void expire(String key, int timeout);

    /**
     * 方法注释
     *
     * @param key
     * @param timeout
     * @param unit
     * @author WANGWEI
     */
    public void expire(String key, final long timeout, final TimeUnit unit);

    /**
     * 方法注释
     *
     * @param key
     * @param c
     * @param timeout
     * @return
     * @author WANGWEI
     */
    public <T> T get(String key, Class<T> c, int timeout);

    /**
     * 方法注释
     *
     * @param key
     * @param c
     * @return
     * @author WANGWEI
     */
    public <T> T get(String key, Class<T> c);

    /**
     * hash get
     *
     * @param <T>
     * @param key
     * @param subkey
     * @param c
     * @return
     */
    public <T> T get(String key, String subkey, Class<T> c);

    /**
     * 方法注释
     *
     * @param key
     * @author WANGWEI
     */
    public void delete(String key);

    public void delete(String key, String subkey);

    /**
     * 方法注释
     *
     * @param channel
     * @param message
     * @author WANGWEI
     */
    public void convertAndSend(String channel, Object message);

    /**
     * (在key不存在时,创建并设置value 返回true; key存在时,会反回false)
     *
     * @param key
     * @param value
     * @param timeout
     * @return
     * @author WANGWEI
     */
    public Boolean setIfAbsent(String key, String value, int timeout);

    /**
     * 获取失效时间
     *
     * @param key
     * @param timeUnit
     * @return
     * @author WANGWEI
     */
    public Long getExpire(String key, TimeUnit timeUnit);

}