package cn.com.qmth.examcloud.web.task;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 任务调度抽象类
 *
 * @author WANGWEI
 * @date 2018年3月7日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public abstract class AbstractTask implements Task {

    protected static final Logger LOGGER = LoggerFactory.getLogger(AbstractTask.class);

    public abstract TaskTracker getTaskTracker();

    public abstract void run(ScheduleJob scheduleJob) throws Exception;

    @Override
    public void execute(ScheduleJob scheduleJob, String traceId) {
        try {
            getTaskTracker().start(scheduleJob, traceId);
        } catch (Exception e) {
            LOGGER.error("[JOB] start err.. jobId={}", scheduleJob.getJobId(), e);
        }

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("[JOB] start.. jobId={} jobImpl={}", scheduleJob.getJobId(), scheduleJob.getSpringBean());
        }

        try {
            run(scheduleJob);
        } catch (Exception e) {
            try {
                getTaskTracker().whenException(scheduleJob, traceId, e);
            } catch (Exception ex) {
                LOGGER.error("[JOB] whenException.. jobId={}", scheduleJob.getJobId(), ex);
            }

            LOGGER.error("[JOB] run err.. jobId={}", scheduleJob.getJobId(), e);
            throw new ExamCloudRuntimeException(e);
        }

        try {
            getTaskTracker().onEnd(scheduleJob, traceId);
        } catch (Exception e) {
            LOGGER.error("[JOB] end err.. jobId={}", scheduleJob.getJobId(), e);
        }

        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("[JOB] end.. jobId={} jobImpl={}", scheduleJob.getJobId(), scheduleJob.getSpringBean());
        }
    }

}
